==> series-1.0.1.tar.gz: series-1.0.1/series/subsync/__init__.py <==
__copyright__ = """ Copyright (c) 2011-2012 Torsten Schmits

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>.
"""

import re, os
from cStringIO import StringIO

from lxml import etree

import requests

from tek import logger, ConfigClient, Configurations
from tek.config import lazy_configurable
from tek.user_input import CheckboxList

from series import SubtitleMetadata, latest_season_dir

from series.subsync.config import reset_config
from series.subsync.errors import NoSubsForEpisode

reset_config()

class CookieHTTPHandler(object):

    def __init__(self, cookies):
        self._cookies = cookies

    def get(self, url):
        return requests.get(url, cookies=self._cookies).content

@lazy_configurable(subsync=['base_url', 'cookies'])
class SubtitleHTTPHandler(CookieHTTPHandler):

    def __init__(self):
        cookies = dict([c.split(':') for c in self._cookies])
        super(SubtitleHTTPHandler, self).__init__(cookies)

    def get(self, url):
        url = self._base_url + url
        return super(SubtitleHTTPHandler, self).get(url)

@lazy_configurable(series=['series_dir'], subsync=['base_url', 'cookies'])
class Subtitle(object):

    def __init__(self, info, release, url, hearing_impaired):
        self.metadata = info
        self.release = release
        self._url = url
        self._content = None
        self.hearing_impaired = hearing_impaired
        self._http = SubtitleHTTPHandler()

    @property
    def local_path(self):
        return self.metadata.local_path

    @property
    def content(self):
        if self._content is None:
            self._content = self._http.get(self._url)
        return self._content

    def write(self):
        path = os.path.join(self._series_dir, self.local_path)
        base = os.path.dirname(path)
        if not os.path.isdir(base):
            os.makedirs(base)
        with open(path, 'w') as f:
            f.write(self.content)

class HTMLParser(object):
    xpath_deaf = "descendant::img[@title='Hearing Impaired']"
    xpath_download = "descendant::a[@class='buttonDownload']"
    release_re = re.compile(r'Version ([^,]+),')
    parser = etree.HTMLParser()

    def __init__(self, info):
        self._info = info
        self._http = SubtitleHTTPHandler()

    def subtitles(self):
        url = 'serie/{}/{}/{}/1'.format(*self._info)
        html = self._http.get(url)
        tree = etree.parse(StringIO(html), self.parser)
        if tree.getroot() is None:
            raise NoSubsForEpisode(*self._info)
        epi_divs = tree.xpath("//div[@id='container95m']")
        return filter(None, map(self.subtitle_from_div, epi_divs))

    def subtitle_from_div(self, div):
        def release():
            for line in div.itertext():
                match = self.release_re.search(line)
                if match:
                    return match.group(1).lower()
        def subtitle():
            download = div.xpath(self.xpath_download)[-1]
            return download.attrib['href']
        try:
            hearing_impaired = bool(div.xpath(self.xpath_deaf))
            return Subtitle(self._info, release(), subtitle(), hearing_impaired)
        except IndexError:
            pass

def get_episode(series, season, number, release=None):
    from tek.tools import find
    info = SubtitleMetadata(series, season, number, release)
    parser = HTMLParser(info)
    subs = parser.subtitles()
    if not subs:
        raise NoSubsForEpisode(series, season, number)
    same_release = lambda sub: sub.release == release
    if release is not None:
        matching = filter(same_release, subs)
        if matching:
            subs = matching
    not_deaf = lambda sub: not sub.hearing_impaired
    return find(not_deaf, subs) or subs[0]

def write_subs(subs):
    from operator import attrgetter
    input = CheckboxList(map(attrgetter('local_path'), subs),
                         [True] * len(subs),
                         text_pre=['Downloading these subtitles:'],
                         text_post=['Write to series dir?'])
    write = input.read()
    for sub, doit in zip(subs, write):
        if doit:
            sub.write()
    return subs

def subsync_episode(metadata, write=True):
    try:
        sub = get_episode(*metadata.all)
        if write:
            sub.write()
        return sub
    except NoSubsForEpisode:
        pass
    
def subsync_dir(dir, write=True):
    from series import dir_info, episode_number, get_release
    Configurations.parse_cli()
    subs = []
    series_name, season = dir_info(dir)
    if season is None:
        logger.error('Not a season dir!')
    else:
        subs_dir = os.path.join(dir, 'sub')
        if not os.path.isdir(subs_dir):
            os.makedirs(subs_dir)
        present_vids = os.listdir(dir)
        present_subs = os.listdir(subs_dir)
        vid_epis = filter(None, map(episode_number, present_vids))
        sub_epis = filter(None, map(episode_number, present_subs))
        vid_max = max(vid_epis) if vid_epis else 0
        sub_max = max(sub_epis) if sub_epis else 0
        subs = []
        try:
            for number in xrange(sub_max+1, vid_max+1):
                release = get_release(series_name, season, number)
                subs.append(get_episode(series_name, season, number, release))
        except NoSubsForEpisode:
            pass
        if subs and write:
            write_subs(subs)
    return subs
    
def subsync_cwd():
    return subsync_dir(os.getcwd())

def subsync_auto():
    monitor = ConfigClient('series')('monitor')
    def sync():
        for dir in map(latest_season_dir, monitor):
            yield subsync_dir(dir, write=False)
    subs = sum(sync(), [])
    if subs:
        write_subs(subs)

==> series-1.1.1.tar.gz: series-1.1.1/series/subsync/__init__.py <==
import re
import os
from io import StringIO
from urllib.parse import urljoin
from operator import attrgetter

from lxml import etree

import requests

from tek import logger, ConfigClient, Configurations
from tek.config import lazy_configurable, ConfigError
from tek.user_input import CheckboxList
from tek.tools import parallel_map

from series import SubtitleMetadata, latest_season_dir
from series import dir_info, episode_number, get_release

from series.subsync.config import reset_config
from series.subsync.errors import NoSubsForEpisode

reset_config()


class CookieHTTPHandler(object):

    def __init__(self, cookies):
        self._cookies = cookies

    def get(self, url):
        return requests.get(url, cookies=self._cookies).text


@lazy_configurable(subsync=['base_url', 'cookies'])
class SubtitleHTTPHandler(CookieHTTPHandler):

    def __init__(self):
        cookies = dict([c.split(':') for c in self._cookies])
        super(SubtitleHTTPHandler, self).__init__(cookies)

    def get(self, url):
        if self._base_url is None:
            raise ConfigError('subsync: config option "base_url" must be set!')
        url = urljoin(self._base_url, url)
        return super(SubtitleHTTPHandler, self).get(url)


@lazy_configurable(series=['series_dir'], subsync=['base_url', 'cookies'])
class Subtitle(object):

    def __init__(self, info, release, url, hearing_impaired):
        self.metadata = info
        self.release = release
        self._url = url
        self._content = None
        self.hearing_impaired = hearing_impaired
        self._http = SubtitleHTTPHandler()

    @property
    def local_path(self):
        return self.metadata.local_path

    @property
    def content(self):
        if self._content is None:
            self._content = self._http.get(self._url)
        return self._content

    def write(self):
        path = os.path.join(self._series_dir, self.local_path)
        base = os.path.dirname(path)
        if not os.path.isdir(base):
            os.makedirs(base)
        with open(path, 'w') as f:
            f.write(self.content)


class HTMLParser(object):
    xpath_deaf = "descendant::img[@title='Hearing Impaired']"
    xpath_download = "descendant::a[@class='buttonDownload']"
    release_re = re.compile(r'Version ([^,]+),')
    parser = etree.HTMLParser()

    def __init__(self, info):
        self._info = info
        self._http = SubtitleHTTPHandler()

    def subtitles(self):
        url = 'serie/{}/{}/{}/1'.format(*self._info)
        html = str(self._http.get(url))
        tree = etree.parse(StringIO(html), self.parser)
        if tree.getroot() is None:
            raise NoSubsForEpisode(*self._info)
        epi_divs = tree.xpath("//div[@id='container95m']")
        return [_f for _f in map(self.subtitle_from_div, epi_divs) if _f]

    def subtitle_from_div(self, div):

        def release():
            for line in div.itertext():
                match = self.release_re.search(line)
                if match:
                    return match.group(1).lower()

        def subtitle():
            download = div.xpath(self.xpath_download)[-1]
            return download.attrib['href']
        try:
            hearing_impaired = bool(div.xpath(self.xpath_deaf))
            return Subtitle(self._info, release(), subtitle(),
                            hearing_impaired)
        except IndexError:
            pass


def get_episode(series, season, number, release=None):
    from tek.tools import find
    info = SubtitleMetadata(series, season, number, release)
    parser = HTMLParser(info)
    subs = parser.subtitles()
    if not subs:
        raise NoSubsForEpisode(series, season, number)
    same_release = lambda sub: sub.release == release
    if release is not None:
        matching = list(filter(same_release, subs))
        if matching:
            subs = matching
    not_deaf = lambda sub: not sub.hearing_impaired
    return find(not_deaf, subs) or subs[0]


def write_subs(subs):
    input = CheckboxList(list(map(attrgetter('local_path'), subs)),
                         [True] * len(subs),
                         text_pre=['Downloading these subtitles:'],
                         text_post=['Write to series dir?'])
    write = input.read()

    def process(args):
        sub, doit = args
        if doit:
            sub.write()

    return parallel_map(process, zip(subs, write))


def subsync_episode(metadata, write=True):
    try:
        sub = get_episode(*metadata.all)
        if write:
            sub.write()
        return sub
    except NoSubsForEpisode:
        pass


def target_episodes(vid_epis, sub_epis):
    conf = ConfigClient('subsync')
    targets = conf('episodes')
    if not targets:
        if conf('only_latest'):
            vid_max = max(vid_epis) if vid_epis else 0
            sub_max = max(sub_epis) if sub_epis else 0
            targets = range(sub_max+1, vid_max+1)
        else:
            targets = set(vid_epis) - set(sub_epis)
    return sorted(targets)


def subsync_dir(_dir, write=True):
    Configurations.parse_cli()
    subs = []
    series_name, season = dir_info(_dir)
    if season is None:
        logger.error('Not a season dir!')
    else:
        subs_dir = os.path.join(_dir, 'sub')
        if not os.path.isdir(subs_dir):
            os.makedirs(subs_dir)
        present_vids = os.listdir(_dir)
        present_subs = os.listdir(subs_dir)
        vid_epis = [_f for _f in map(episode_number, present_vids) if _f]
        sub_epis = [_f for _f in map(episode_number, present_subs) if _f]
        targets = target_episodes(vid_epis, sub_epis)

        def process(number):
            try:
                release = get_release(series_name, season, number)
                return get_episode(series_name, season, number, release)
            except NoSubsForEpisode:
                logger.warn('No subs found for episode {}!'.format(number))

        subs = [sub for sub in parallel_map(process, targets) if sub is not
                None]
        if subs and write:
            write_subs(subs)
    return subs


def subsync_cwd():
    return subsync_dir(os.getcwd())


def subsync_auto():
    monitor = ConfigClient('series')('monitor')

    def sync():
        for _dir in map(latest_season_dir, monitor):
            yield subsync_dir(_dir, write=False)

    subs = sum(sync(), [])
    if subs:
        write_subs(subs)
